// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockdaemon.h 97 2008-01-24 18:45:06Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie Gmbh.
**
** Author: Wolfgang Glas
**
** A class, that encapsulates a full-fledged UNIX-daemon.
**
*****************************************************************************/

#ifndef FANCYSOCK_DAEMON_H
#define FANCYSOCK_DAEMON_H

#include <iosfwd>
#include <sys/types.h>
#include <fancysock.h>

#ifdef WIN32
extern "C" {
  typedef int fancysock_main_func_t(int argc, char** argv);
  int FancySockServiceMain(int argc, char** argv);
}

#define FANCYSOCK_DAEMON_MAIN_DISPATCH(a,b,c)                  \
APIENTRY WinMain( HINSTANCE instance, HINSTANCE prevInstance, \
		      LPSTR  cmdParam, int cmdShow ) \
{ return Fancy::Sock::ServiceDispatch(cmdParam,c,FancySockServiceMain); } \
int FancySockServiceMain(a,b)
#define FANCYSOCK_DAEMON_MAIN(a,b)  FANCYSOCK_DAEMON_MAIN_DISPATCH(a,b,false)
#define FANCYSOCK_DAEMON_MAIN_INTERACTIVE(a,b) FANCYSOCK_DAEMON_MAIN_DISPATCH(a,b,true)
#else
#define FANCYSOCK_DAEMON_MAIN(a,b) main(a,b)
#define FANCYSOCK_DAEMON_MAIN_INTERACTIVE(a,b) main(a,b)
#endif

namespace Fancy {
  namespace Sock {
#ifdef WIN32
    FANCYSOCK_API int ServiceDispatch(char *cmdParam, bool interactive,
                                      fancysock_main_func_t *main_func);
#endif

    class EmailClientTransport;
    class DaemonPrivate;

    class FANCYSOCK_API Daemon
    {
    public:
      Daemon (const char *name);
      virtual ~Daemon ();

      void killInstance();

      virtual void getStatus(std::ostream &os) const;
      void printInstanceStatus();

      void mailStatus(const std::string &info,
                      EmailClientTransport *transport) const;

      void instantiate(const char *su_user=0, unsigned timeout=10);

      int fork(int nprocesses);

      virtual void leaveConsole();

      bool consoleLeft() const;

#ifndef WIN32
      pid_t find_instance();
#endif

    private:
      DaemonPrivate *d;
    };
  }
}
 
#endif
