// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockhttptransport.h 89 2007-12-18 20:28:22Z wglas $
**
** Copyright (C) 2005 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A minimal HTTP/1.1 implementation for transporting arbitrary documents.
** 
*****************************************************************************/

#ifndef FANCYSOCK_HTTP_TRANSPORT_H
#define FANCYSOCK_HTTP_TRANSPORT_H

#include <fancysocktransport.h>
#include <fancysockexception.h>

namespace Fancy {
  namespace Sock {

    class FANCYSOCK_API HttpException : public Exception
    { 
    public:
      HttpException (fancylog_service *service,
                     int facility,
                     int priority,
                     int reason,
                     const char *fmt, ...) throw()
#ifdef __GNUG__
        __attribute__((format(printf,6,7)))
#endif
        ;

      HttpException (int facility,
                     int priority,
                     int reason,
                     const char *fmt, ...) throw()
#ifdef __GNUG__
        __attribute__((format(printf,5,6)))
#endif
        ;

      ~HttpException() throw();

      int getReason() const throw();

    private:
      int r;
    };

    class FANCYSOCK_API HttpServerTransport : public Transport
    {
    public:
      HttpServerTransport(Device *d,
                          int bufsize = 16382);
    
      ~HttpServerTransport();
    
      void openWriteDoc(const std::string &mimetype);
      std::string openReadDoc();

      int write (const char *buf, int count);
      int read (char *buf, int count);

      void closeWriteDoc();
      void closeReadDoc();

      bool haveV10Client() const;
      const std::string &getRequestMethod() const;
      const std::string &getRequestUri() const;
      const std::string &getRequestQuery() const;
      const std::string &getContentEncoding() const;
    
      void writeError(int reason, const std::string &msg);
      void writeError(const HttpException &e);

      int getResponseReason() const;
      void setResponseReason(int r);

      int getResponseTTL() const;
      void setResponseTTL(int r);

    private:
      int read_line(std::string &line);
      int write_line(const std::string &line);

      HttpServerTransport (const HttpServerTransport &);
      HttpServerTransport &operator=(const HttpServerTransport &);

      int bufsize;
      unsigned char *ibuf;
      unsigned char *g_ptr;
      unsigned char *g_end;
      unsigned char *obuf;
      unsigned char *p_start;
      unsigned char *p_ptr;
      bool chunked;
      bool http_10_client;
      int chunk_size;

      std::string method;
      std::string uri;
      std::string query;
      std::string content_enc;

      int reason;
      int ttl;
    };
  }
}

#endif // ! FANCYSOCK_HTTP_TRANSPORT_H
