// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockserver.h 89 2007-12-18 20:28:22Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Highlevel socket server class for handling multiple connections
** in seperate threads.
** 
*****************************************************************************/

#ifndef FANCYSOCK_SERVER_H
#define FANCYSOCK_SERVER_H

#include <string>
#include <iosfwd>
#include <fancythread.h>
#include <fancysock.h>

namespace Fancy {
  namespace Sock {

    // fwd decls.
    class Address;
    class Device;

    class ServerPrivate;
    class Server;

    class FANCYSOCK_API ClientHandler : public Thread
    {
    public:
      ClientHandler(Server *s);
      virtual ~ClientHandler();

      virtual void statusInfo(std::ostream &os) const;
    protected:

      virtual void connectionLoop(Device *) = 0;
      Server *getServer() const;

      void run();

    private:
      Server *server;
    };

    class FANCYSOCK_API Server
    {
    public:
      Server (Device *dev);
      Server ();

      virtual ~Server();

      class FANCYSOCK_API ClientConnection
      {
      public:
        ClientConnection(Server *s);
        ~ClientConnection();

        Device *takeConnection();
      private:
        ServerPrivate *d;
        bool active;
      };

      void setGCInterval(int msec);

      void bind (const Address &address, int max_connections);

      void listenLoop();

      static void staticStatusInfo(std::ostream &os);

      virtual void statusInfo(std::ostream &os) const;

    protected:
      virtual bool mayTerminate();
      virtual void garbageCollect();
      virtual ClientHandler *createClientHandler() = 0;

    private:
      Server (const Server &);
      Server &operator=(const Server &);

      ServerPrivate *d;
    };

  }
}

#endif // ! FANCYSOCK_SERVER_H
