// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysocktransport.h 89 2007-12-18 20:28:22Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps a document transport layer on top of a device.
** 
*****************************************************************************/

#ifndef FANCYSOCK_TRANSPORT_H
#define FANCYSOCK_TRANSPORT_H

#include <string>
#include <fancysock.h>

namespace Fancy {
  namespace Sock {

    class Device;

    class FANCYSOCK_API Transport
    {
    public:
      Transport(Device *d=0);

      virtual ~Transport();

      Device *getDevice() const;

      virtual void openWriteDoc(const std::string &mimetype);
      virtual std::string openReadDoc() = 0;

      virtual int write (const char *buf, int count) = 0;
      virtual int read (char *buf, int count) = 0;

      virtual void closeWriteDoc();
      virtual void closeReadDoc();

    protected:
      void setDevice (Device *d);

    private:
      Transport (const Transport &);
      Transport &operator=(const Transport &);

      Device *device;
    };

    class FANCYSOCK_API RawTransport : public Transport
    {
    public:
      RawTransport (Device *d,
                    int bufsize = 16384);
    
      ~RawTransport();
    
      std::string openReadDoc();

      int write (const char *buf, int count);
      int read (char *buf, int count);

      void closeWriteDoc();

    private:
      RawTransport (const RawTransport &);
      RawTransport &operator=(const RawTransport &);

      int bufsize;
      unsigned char *ibuf;
      unsigned char *g_ptr;
      unsigned char *g_end;
      unsigned char *obuf;
      unsigned char *p_ptr;
    };
  }
}

#endif // ! FANCYSOCK_TRANSPORT_H
